//
//  FileTest.cpp
//
//  Created  by Setsu on 5/18/2010.
//  Modified by Setsu on May 30, 2013.
//  Copyright 2010-2013 RoundSquare Inc. All rights reserved.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//

//----------------------------------------
//	include
//----------------------------------------
#include <Poco/File.h>
#include <Poco/Path.h>
#include <Poco/Format.h>
#include <Poco/DateTimeFormatter.h>
#include <Poco/Timezone.h>
#include <Poco/String.h>
#include <Poco/Exception.h>

#include <string>

#include "ScopedLogMessage.h"
#include "PrepareConsoleLogger.h"

//----------------------------------------
//	Macro
//----------------------------------------
#define ShowBoolWithException(msg, text, result)								\
	try																			\
	{																			\
		msg.Message(Poco::format(text+": %s", IsTrue(result)));					\
	}																			\
	catch(const Poco::Exception& exp)											\
	{																			\
		msg.Message(Poco::format(text+": EXCEPTION [%s]", exp.displayText()));	\
	}																			\

#define ShowSizeWithException(msg, text, size)									\
	try																			\
	{																			\
		msg.Message(Poco::format(text+": %z", std::size_t(size)));				\
	}																			\
	catch(const Poco::Exception& exp)											\
	{																			\
		msg.Message(Poco::format(text+": EXCEPTION [%s]", exp.displayText()));	\
	}																			\

#define ShowDateWithException(msg, text, timestamp)								\
	try																			\
	{																			\
		ShowDateTime(msg, timestamp, text+": %s");								\
	}																			\
	catch(const Poco::Exception& exp)											\
	{																			\
		msg.Message(Poco::format(text+": EXCEPTION [%s]", exp.displayText()));	\
	}																			\

//----------------------------------------
//	IsTrue
//----------------------------------------
inline std::string IsTrue(bool isTrue)
{
	return std::string(isTrue ? "true":"false");
};

//----------------------------------------
//	ShowDateTime
//----------------------------------------
void ShowDateTime(ScopedLogMessage& msg, const Poco::Timestamp& timestamp, const std::string& fmt)
{
	Poco::DateTime createdDateTime(timestamp);
	createdDateTime.makeLocal(Poco::Timezone::tzd());
	msg.Message(Poco::format(
					Poco::format(fmt, Poco::DateTimeFormatter::format(createdDateTime, "%w %b %e %H:%M:%S %%s %Y"))
				,	Poco::Timezone::name()));
};

//----------------------------------------
//	ShowFileCharacteristics
//----------------------------------------
void ShowFileCharacteristics(ScopedLogMessage& msg, const Poco::File& file)
{
	ShowBoolWithException(msg, std::string("    isDevice()        "), file.isDevice());
	ShowBoolWithException(msg, std::string("    isDirectory()     "), file.isDirectory());
	ShowBoolWithException(msg, std::string("    isFile()          "), file.isFile());
	ShowBoolWithException(msg, std::string("    isHidden()        "), file.isHidden());
	ShowBoolWithException(msg, std::string("    isLink()          "), file.isLink());
	ShowBoolWithException(msg, std::string("    canExecute()      "), file.canExecute());
	ShowBoolWithException(msg, std::string("    canRead()         "), file.canRead());
	ShowBoolWithException(msg, std::string("    canWrite()        "), file.canWrite());
	ShowSizeWithException(msg, std::string("    getSize()         "), file.getSize());
	ShowDateWithException(msg, std::string("    created()         "), file.created());
	ShowDateWithException(msg, std::string("    getLastModified() "), file.getLastModified());
};

//----------------------------------------
//	main
//----------------------------------------
int main(int /*argc*/, char** /*argv*/)
{
	PrepareConsoleLogger logger(Poco::Logger::ROOT, Poco::Message::PRIO_INFORMATION);

	ScopedLogMessage msg("FileTest ", "start", "end");

	Poco::File dir(Poco::Path("testDir1/testDir2/testDir3", Poco::Path::PATH_UNIX));
	dir.createDirectories();
	if(dir.exists())
	{
		msg.Message(Poco::format("  %s created", dir.path()));
		ShowFileCharacteristics(msg, dir);
	}
	msg.Message("");

	Poco::File file(Poco::Path(Poco::Path(dir.path()), "testFile.txt"));
	file.createFile();
	if(file.exists())
	{
		msg.Message(Poco::format("  %s created", file.path()));
		ShowFileCharacteristics(msg, file);
		msg.Message("");

		msg.Message(Poco::format("  change %s characteristics", file.path()));
		file.setExecutable(true);
		file.setReadOnly(true);
		file.setWriteable(false);
		ShowFileCharacteristics(msg, file);
		msg.Message("");

		try
		{
			file.renameTo(Poco::replace(file.path(), "testFile", "renamedFile"));
			msg.Message(Poco::format("  %s renamed", file.path()));
		}
		catch(const Poco::Exception& exp)
		{
			msg.Message(Poco::format("  EXCEPTION [%s]", exp.displayText()));
		}
	}
	msg.Message("");

#if defined(POCO_OS_FAMILY_WINDOWS)
	file.setWriteable(true);
#endif
	try
	{
		Poco::File topDir(Poco::Path("testDir1", Poco::Path::PATH_UNIX));
		topDir.remove(true);
		msg.Message(Poco::format("  %s %s", topDir.path(), std::string(topDir.exists() ? "exists":"removed")));
	}
	catch(const Poco::Exception& exp)
	{
		msg.Message(Poco::format("  EXCEPTION [%s]", exp.displayText()));
	}

	return 0;
}
